@@ -436,6 +436,20 @@ std::pair<Text, Text> PinyinEngine::preedit(InputContext *inputContext) const {
     return {std::move(clientPreedit), std::move(preedit)};
 }
 
+std::string
+PinyinEngine::preeditCommitString(InputContext *inputContext) const {
+    auto *state = inputContext->propertyFor(&factory_);
+    // Use const ref to avoid accidentally change anything.
+    const auto &context = state->context_;
+
+    const auto &userInput = context.userInput();
+    const auto selectedLength = context.selectedLength();
+
+    return context.selectedSentence() +
+           userInput.substr(selectedLength,
+                            userInput.length() - selectedLength);
+}
+
 void PinyinEngine::updatePreedit(InputContext *inputContext) const {
     auto &inputPanel = inputContext->inputPanel();
     auto [clientPreedit, preedit] = this->preedit(inputContext);
@@ -1019,12 +1033,27 @@ void PinyinEngine::activate(const fcitx::InputMethodEntry &entry,
 void PinyinEngine::deactivate(const fcitx::InputMethodEntry &entry,
                               fcitx::InputContextEvent &event) {
     auto *inputContext = event.inputContext();
-    if (event.type() == EventType::InputContextSwitchInputMethod) {
-        auto *state = inputContext->propertyFor(&factory_);
-        if (!state->context_.empty()) {
-            inputContext->commitString(state->context_.userInput());
+    do {
+        if (event.type() != EventType::InputContextSwitchInputMethod) {
+            break;
         }
-    }
+        auto *state = inputContext->propertyFor(&factory_);
+        if (state->context_.empty()) {
+            break;
+        }
+
+        switch (*config_.switchInputMethodBehavior) {
+        case SwitchInputMethodBehavior::CommitPreedit:
+            inputContext->commitString(preeditCommitString(inputContext));
+            break;
+        case SwitchInputMethodBehavior::CommitDefault: {
+            inputContext->commitString(state->context_.sentence());
+            break;
+        }
+        case SwitchInputMethodBehavior::Clear:
+            break;
+        }
+    } while (0);
     reset(entry, event);
 }
 
@@ -1674,7 +1703,7 @@ void PinyinEngine::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
             event.filterAndAccept();
         } else if (event.key().sym() == FcitxKey_Return ||
                    event.key().sym() == FcitxKey_KP_Enter) {
-            inputContext->commitString(state->context_.userInput());
+            inputContext->commitString(preeditCommitString(inputContext));
             state->context_.clear();
             event.filterAndAccept();
         } else if (int idx =
