@@ -12,6 +12,7 @@
 #include <fcitx-utils/i18n.h>
 #include <fcitx-utils/log.h>
 #include <fcitx/candidatelist.h>
+#include <libime/core/prediction.h>
 #include <libime/core/userlanguagemodel.h>
 #include <libime/table/tablebaseddictionary.h>
 #include <tuple>
@@ -81,6 +82,9 @@ FCITX_CONFIGURATION(
          Key(FcitxKey_9), Key(FcitxKey_0)}};
     Option<int, IntConstrain> pageSize{this, "PageSize", _("Page size"), 5,
                                        IntConstrain(3, 10)};
+    SubConfigOption punctuationMap{this, "TableGlobal",
+                                   _("Table Global Options"),
+                                   "fcitx://config/addon/table"};
     // Fcitx 4 default behavior.
     Option<bool> commitAfterSelect{this, "CommitAfterSelect",
                                    _("Commit after auto select candidates"),
